
/**************************************************************************************************
*
*  SailingSensoreNode
*
*   Version:      1.3.0 - April 2009
*   Author:       Christoph Wartmann / chair for caad - ETH Zrich  /  wartmann[at].arch.ethz.ch
*                 Etienne Ribeiro    / tutorial assistant caad      /  eribeiro[at]ethz.ch
*
*   Description:  Mesures values from sensors and sends them trough xbee. We're using modStringSerial
*                 as Protocol to send messages in ASCII mode.
*                 Note: The aneometer sensors delays the execution of the hole program. Meaning if
*                 there is no wind blowing, no values will be sent.
*
*   Used Modules: modAneometer: Mesures wind speed
*                 modPressure: Mesures Pressure
*                 modStringSerial: Protocol to transmit values
*
*   Hardware:     Arduino Pro Mini
*                 XBee in RS232 Mode
*                 Pressure Sensor SCP 1000
*                 Aneometer Sensor using induction to calculate wind speed
*
*   Use commands: m = message (general message)
*                 s = sensor value (aneometer, temperature, pressure)
*                 A = set aneometer interval values 100ms to 1000ms
*                 B = get aneometer interval values
*                 P = set pressure diff [Pa]
*                 Q = get pressure diff
*
**************************************************************************************************/


#include <EEPROM.h>


// EEPROM
//   set values to 0 to get default from EEPROM, otherwise EEPROM will be overwritten

int NODE_ID = -1; // -1 read from eeprom, 0 to 256 DON'T USE values 10 and 13 !!!!
int Aneometer_Intervall = 400; // 100 to 1000 ms 
int Pressure_Diff = -1; // -1 read from eeprom, 0 to 32000 Pa


// Const

char VERSION[] = "1.3.0";
#define aneometerSignalIn 9
#define ledPin 13


// Var

boolean startupDone;


//
// Setup

void setup() {
      
  // LED (2 * blink)
  pinMode(ledPin, OUTPUT);
  digitalWrite(ledPin, HIGH);
  delay(200);
  digitalWrite(ledPin, LOW);
  delay(200);
  digitalWrite(ledPin, HIGH);


  // R/W to EEPTOM
  boolean doWrite = false;
  // NODE_ID
  if (NODE_ID == -1)
    NODE_ID = EEPROM.read(10);
  if (EEPROM.read(10) != NODE_ID) {
    EEPROM.write (10, NODE_ID);
    doWrite = true;
  }
  // Pressure_Diff
  if (Pressure_Diff == -1) {
    Pressure_Diff = EEPROM.read(20) << 8;
    Pressure_Diff += EEPROM.read(21);  
  }
  int tmpPressure_Diff;
  tmpPressure_Diff = EEPROM.read(20) << 8;
  tmpPressure_Diff += EEPROM.read(21); 
  if (Pressure_Diff != tmpPressure_Diff) {
    EEPROM.write (20, (char) Pressure_Diff>>8);
    EEPROM.write (21, (char) Pressure_Diff);  
    doWrite = true;
  }
    
  
  // Serial
  Serial_Initialize (9600);
  Serial_setID (NODE_ID); // ID will be sent
  Serial_sendMessage ('m', "Setup begin");
  Serial_sendMessage ('m', "Firmware Verison ", VERSION);
  Serial_sendMessage ('m', "My ID ", NODE_ID);
  Serial_sendMessage ('m', "Pressure_Diff ", Pressure_Diff);
  if (doWrite == true)
    Serial_sendMessage ('m', "Wrote to EEPROM");  
  if (NODE_ID == 10 || NODE_ID == 13)
    Serial_sendMessage ('m', " CHANGE ID! ID 10 or 13 is not recommended to use!", VERSION);
  
  
  // Pressure
  //  -> Operationmode: 0: High speed (15bit, 9Hz), 1: High resolution (17bit, 1.8Hz), 2: Ultra low power (15bit, 1Hz), [3 Low poser but triggerd (15bit to 17bit, 1.8Hz) not supported on Sparkfun Breakout]
  byte operationmode = 1;
  startupDone = true;
  if (scp1000_AppendSensor (10, 11, 12, 13, 14, operationmode) == false) {
  
    Serial_sendMessage ('m', "Failed to startup Pressure Sensor");
    startupDone = false;
  
  }
  Serial_sendMessage ('m', "Setup done");
  
  
  // Send Aneometer Intervall
  Serial_sendMessage ('B', Aneometer_Intervall);

}




//
// Loop

void loop() {

  if (startupDone) {  

    // Aneometer
    int HzCounter = Aneo_getSpeedHZ (aneometerSignalIn, Aneometer_Intervall); // Aneometer_Intervall values 0 to 1000 ms
    // Temperature
    int sign;
    int temp1;
    int temp2;
    scp1000_parseDegrees(scp1000_Read('t'), &sign, &temp1, &temp2);
    long temperature = sign * (temp1 * 100 + temp2);
    // Pressure
    long pressure = scp1000_parsePa(scp1000_Read('p'));
    
    
    // Send message
    Serial_sendMessage ('s', HzCounter, ":", temperature, ":", pressure + Pressure_Diff, ":", (int) (millis() / 100));

  }
  
  
  // Check Serial
  Serial_check ();
  
   
  // Take a break
  delay(100);
  
}





// SerialEvent

void SerialEvent (char cmd, char* message) {
  
  if (cmd == 'A') // set aneometer intervall value
    Aneometer_Intervall = Serial_parseInteger (message);
  else if (cmd == 'B') // send aneometer intervall
    Serial_sendMessage ('B', Aneometer_Intervall);
  else if (cmd == 'P') {  // set pressure diff
  
    // Parse sent pressure value
    Pressure_Diff = Serial_parseInteger (message);
    
    
    // write to eeprom if not already done
    int tmpPressure_Diff;
    tmpPressure_Diff = EEPROM.read(20) << 8;
    tmpPressure_Diff += EEPROM.read(21); 
    if (Pressure_Diff != tmpPressure_Diff) {
      EEPROM.write (20, (char) Pressure_Diff>>8);
      EEPROM.write (21, (char) Pressure_Diff);  
      Serial_sendMessage ('m', "Wrote to EEPROM");
    }
    
  }
  else if (cmd == 'Q') // send pressure diff
    Serial_sendMessage ('Q', Pressure_Diff);
  
}
